package com.sun.imageio.plugins.jpeg;

import java.awt.color.ICC_Profile;
import javax.imageio.stream.ImageOutputStream;
import java.awt.Point;
import javax.imageio.plugins.jpeg.JPEGHuffmanTable;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import javax.imageio.metadata.IIOInvalidTreeException;
import javax.imageio.metadata.IIOMetadataNode;
import org.w3c.dom.Node;
import java.util.ListIterator;
import java.util.Iterator;
import java.awt.image.ColorModel;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ColorSpace;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.ImageWriteParam;
import java.io.IOException;
import javax.imageio.IIOException;
import javax.imageio.stream.ImageInputStream;
import java.util.ArrayList;
import java.util.List;
import javax.imageio.metadata.IIOMetadata;

public class JPEGMetadata extends IIOMetadata implements Cloneable
{
    private static final boolean debug = false;
    private List resetSequence;
    private boolean inThumb;
    private boolean hasAlpha;
    List markerSequence;
    final boolean isStream;
    private boolean transparencyDone;
    
    JPEGMetadata(boolean isStream, boolean inThumb) {
        super(true, "javax_imageio_jpeg_image_1.0", "com.sun.imageio.plugins.jpeg.JPEGImageMetadataFormat", null, (String[])null);
        this.resetSequence = null;
        this.inThumb = false;
        this.markerSequence = new ArrayList();
        this.inThumb = inThumb;
        this.isStream = isStream;
        if (isStream) {
            this.nativeMetadataFormatName = "javax_imageio_jpeg_stream_1.0";
            this.nativeMetadataFormatClassName = "com.sun.imageio.plugins.jpeg.JPEGStreamMetadataFormat";
        }
    }
    
    JPEGMetadata(boolean b, boolean b2, ImageInputStream imageInputStream, JPEGImageReader jpegImageReader) throws IOException {
        this(b, b2);
        JPEGBuffer jpegBuffer = new JPEGBuffer(imageInputStream);
        jpegBuffer.loadBuf(0);
        if ((jpegBuffer.buf[0] & 0xFF) != 0xFF || (jpegBuffer.buf[1] & 0xFF) != 0xD8 || (jpegBuffer.buf[2] & 0xFF) != 0xFF) {
            throw new IIOException("Image format error");
        }
        int i = 0;
        jpegBuffer.bufAvail -= 2;
        jpegBuffer.bufPtr = 2;
        Object o = null;
        while (i == 0) {
            jpegBuffer.loadBuf(1);
            jpegBuffer.scanForFF(jpegImageReader);
            switch (jpegBuffer.buf[jpegBuffer.bufPtr] & 0xFF) {
                case 0: {
                    --jpegBuffer.bufAvail;
                    ++jpegBuffer.bufPtr;
                    break;
                }
                case 192:
                case 193:
                case 194: {
                    if (b) {
                        throw new IIOException("SOF not permitted in stream metadata");
                    }
                    o = new SOFMarkerSegment(jpegBuffer);
                    break;
                }
                case 219: {
                    o = new DQTMarkerSegment(jpegBuffer);
                    break;
                }
                case 196: {
                    o = new DHTMarkerSegment(jpegBuffer);
                    break;
                }
                case 221: {
                    o = new DRIMarkerSegment(jpegBuffer);
                    break;
                }
                case 224: {
                    jpegBuffer.loadBuf(8);
                    byte[] buf = jpegBuffer.buf;
                    int bufPtr = jpegBuffer.bufPtr;
                    if (buf[bufPtr + 3] == 74 && buf[bufPtr + 4] == 70 && buf[bufPtr + 5] == 73 && buf[bufPtr + 6] == 70 && buf[bufPtr + 7] == 0) {
                        if (this.inThumb) {
                            jpegImageReader.warningOccurred(1);
                            JFIFMarkerSegment o2 = new JFIFMarkerSegment(jpegBuffer);
                            break;
                        }
                        if (b) {
                            throw new IIOException("JFIF not permitted in stream metadata");
                        }
                        if (!this.markerSequence.isEmpty()) {
                            throw new IIOException("JFIF APP0 must be first marker after SOI");
                        }
                        o = new JFIFMarkerSegment(jpegBuffer);
                        break;
                    }
                    else if (buf[bufPtr + 3] == 74 && buf[bufPtr + 4] == 70 && buf[bufPtr + 5] == 88 && buf[bufPtr + 6] == 88 && buf[bufPtr + 7] == 0) {
                        if (b) {
                            throw new IIOException("JFXX not permitted in stream metadata");
                        }
                        if (this.inThumb) {
                            throw new IIOException("JFXX markers not allowed in JFIF JPEG thumbnail");
                        }
                        JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
                        if (jfifMarkerSegment == null) {
                            throw new IIOException("JFXX encountered without prior JFIF!");
                        }
                        jfifMarkerSegment.addJFXX(jpegBuffer, jpegImageReader);
                        break;
                    }
                    else {
                        o = new MarkerSegment(jpegBuffer);
                        o.loadData(jpegBuffer);
                        break;
                    }
                }
                case 226: {
                    jpegBuffer.loadBuf(15);
                    if (jpegBuffer.buf[jpegBuffer.bufPtr + 3] == 73 && jpegBuffer.buf[jpegBuffer.bufPtr + 4] == 67 && jpegBuffer.buf[jpegBuffer.bufPtr + 5] == 67 && jpegBuffer.buf[jpegBuffer.bufPtr + 6] == 95 && jpegBuffer.buf[jpegBuffer.bufPtr + 7] == 80 && jpegBuffer.buf[jpegBuffer.bufPtr + 8] == 82 && jpegBuffer.buf[jpegBuffer.bufPtr + 9] == 79 && jpegBuffer.buf[jpegBuffer.bufPtr + 10] == 70 && jpegBuffer.buf[jpegBuffer.bufPtr + 11] == 73 && jpegBuffer.buf[jpegBuffer.bufPtr + 12] == 76 && jpegBuffer.buf[jpegBuffer.bufPtr + 13] == 69 && jpegBuffer.buf[jpegBuffer.bufPtr + 14] == 0) {
                        if (b) {
                            throw new IIOException("ICC profiles not permitted in stream metadata");
                        }
                        JFIFMarkerSegment jfifMarkerSegment2 = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
                        if (jfifMarkerSegment2 == null) {
                            o = new MarkerSegment(jpegBuffer);
                            o.loadData(jpegBuffer);
                        }
                        else {
                            jfifMarkerSegment2.addICC(jpegBuffer);
                        }
                        break;
                    }
                    else {
                        o = new MarkerSegment(jpegBuffer);
                        o.loadData(jpegBuffer);
                        break;
                    }
                }
                case 238: {
                    jpegBuffer.loadBuf(8);
                    if (jpegBuffer.buf[jpegBuffer.bufPtr + 3] == 65 && jpegBuffer.buf[jpegBuffer.bufPtr + 4] == 100 && jpegBuffer.buf[jpegBuffer.bufPtr + 5] == 111 && jpegBuffer.buf[jpegBuffer.bufPtr + 6] == 98 && jpegBuffer.buf[jpegBuffer.bufPtr + 7] == 101) {
                        if (b) {
                            throw new IIOException("Adobe APP14 markers not permitted in stream metadata");
                        }
                        o = new AdobeMarkerSegment(jpegBuffer);
                        break;
                    }
                    else {
                        o = new MarkerSegment(jpegBuffer);
                        o.loadData(jpegBuffer);
                        break;
                    }
                }
                case 254: {
                    o = new COMMarkerSegment(jpegBuffer);
                    break;
                }
                case 218: {
                    if (b) {
                        throw new IIOException("SOS not permitted in stream metadata");
                    }
                    o = new SOSMarkerSegment(jpegBuffer);
                    break;
                }
                case 208:
                case 209:
                case 210:
                case 211:
                case 212:
                case 213:
                case 214:
                case 215: {
                    ++jpegBuffer.bufPtr;
                    --jpegBuffer.bufAvail;
                    break;
                }
                case 217: {
                    i = 1;
                    ++jpegBuffer.bufPtr;
                    --jpegBuffer.bufAvail;
                    break;
                }
                default: {
                    o = new MarkerSegment(jpegBuffer);
                    o.loadData(jpegBuffer);
                    o.unknown = true;
                    break;
                }
            }
            if (o == null) {
                continue;
            }
            this.markerSequence.add(o);
            o = null;
        }
        jpegBuffer.pushBack();
        if (!this.isConsistent()) {
            throw new IIOException("Inconsistent metadata read from stream");
        }
        return;
    }
    
    JPEGMetadata(ImageWriteParam imageWriteParam, JPEGImageWriter jpegImageWriter) {
        this(true, false);
        JPEGImageWriteParam jpegImageWriteParam = null;
        if (imageWriteParam != null && imageWriteParam instanceof JPEGImageWriteParam) {
            jpegImageWriteParam = (JPEGImageWriteParam)imageWriteParam;
            if (!jpegImageWriteParam.areTablesSet()) {
                jpegImageWriteParam = null;
            }
        }
        if (jpegImageWriteParam != null) {
            this.markerSequence.add(new DQTMarkerSegment(jpegImageWriteParam.getQTables()));
            this.markerSequence.add(new DHTMarkerSegment(jpegImageWriteParam.getDCHuffmanTables(), jpegImageWriteParam.getACHuffmanTables()));
        }
        else {
            this.markerSequence.add(new DQTMarkerSegment(JPEG.getDefaultQTables()));
            this.markerSequence.add(new DHTMarkerSegment(JPEG.getDefaultHuffmanTables(true), JPEG.getDefaultHuffmanTables(false)));
        }
        if (!this.isConsistent()) {
            throw new InternalError("Default stream metadata is inconsistent");
        }
        return;
    }
    
    JPEGMetadata(ImageTypeSpecifier imageTypeSpecifier, ImageWriteParam imageWriteParam, JPEGImageWriter jpegImageWriter) {
        this(false, false);
        boolean n = true;
        boolean n2 = false;
        int n3 = 0;
        boolean n4 = true;
        boolean n5 = false;
        boolean n6 = false;
        boolean n7 = false;
        boolean n8 = true;
        boolean n9 = true;
        float compressionQuality = 0.75f;
        byte[] array = { 1, 2, 3, 4 };
        int n10 = 0;
        ImageTypeSpecifier destinationType = null;
        if (imageWriteParam != null) {
            destinationType = imageWriteParam.getDestinationType();
            if (destinationType != null && imageTypeSpecifier != null) {
                jpegImageWriter.warningOccurred(0);
                destinationType = null;
            }
            if (imageWriteParam.canWriteProgressive() && imageWriteParam.getProgressiveMode() == 1) {
                n6 = true;
                n9 = false;
            }
            if ((imageWriteParam instanceof JPEGImageWriteParam)) {
                JPEGImageWriteParam jpegImageWriteParam = (JPEGImageWriteParam)imageWriteParam;
                if (jpegImageWriteParam.areTablesSet()) {
                    n8 = false;
                    n9 = false;
                    if (jpegImageWriteParam.getDCHuffmanTables().length > 2 || jpegImageWriteParam.getACHuffmanTables().length > 2) {
                        n7 = true;
                    }
                }
                if (n6 == 0 && jpegImageWriteParam.getOptimizeHuffmanTables()) {
                    n9 = false;
                }
            }
            if (imageWriteParam.canWriteCompressed() && imageWriteParam.getCompressionMode() == 2) {
                compressionQuality = imageWriteParam.getCompressionQuality();
            }
        }
        ColorSpace colorSpace = null;
        if (destinationType != null) {
            ColorModel colorModel2 = destinationType.getColorModel();
            n10 = colorModel2.getNumComponents();
            int n12 = colorModel2.getNumColorComponents() != n10;
            int hasAlpha2 = colorModel2.hasAlpha();
            colorSpace = colorModel2.getColorSpace();
            switch (colorSpace.getType()) {
                case 6: {
                    n4 = false;
                    if (!(n12 == 0)) {
                        n = false;
                        break;
                    }
                    break;
                }
                case 13: {
                    if (colorSpace == JPEG.JCS.getYCC()) {
                        n = false;
                        array[0] = 89;
                        array[1] = 67;
                        array[2] = 99;
                        if (!(hasAlpha2 == 0)) {
                            array[3] = 65;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 3: {
                    if (!(n12 == 0)) {
                        n = false;
                        if (hasAlpha2 == 0) {
                            n2 = true;
                            n3 = 2;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 5: {
                    n = false;
                    n2 = true;
                    n4 = false;
                    array[0] = 82;
                    array[1] = 71;
                    array[2] = 66;
                    if (!(hasAlpha2 == 0)) {
                        array[3] = 65;
                        break;
                    }
                    break;
                }
                default: {
                    n = false;
                    n4 = false;
                    break;
                }
            }
        }
        else if (imageTypeSpecifier != null) {
            ColorModel colorModel = imageTypeSpecifier.getColorModel();
            n10 = colorModel.getNumComponents();
            int n11 = colorModel.getNumColorComponents() != n10;
            int hasAlpha = colorModel.hasAlpha();
            colorSpace = colorModel.getColorSpace();
            switch (colorSpace.getType()) {
                case 6: {
                    n4 = false;
                    if (!(n11 == 0)) {
                        n = false;
                        break;
                    }
                    break;
                }
                case 5: {
                    if (!(hasAlpha == 0)) {
                        n = false;
                        break;
                    }
                    break;
                }
                case 13: {
                    n = false;
                    n4 = false;
                    if (colorSpace.equals(ColorSpace.getInstance(1002))) {
                        n4 = true;
                        n2 = true;
                        array[0] = 89;
                        array[1] = 67;
                        array[2] = 99;
                        if (!(hasAlpha == 0)) {
                            array[3] = 65;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 3: {
                    if (!(n11 == 0)) {
                        n = false;
                        if (hasAlpha == 0) {
                            n2 = true;
                            n3 = 2;
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 9: {
                    n = false;
                    n2 = true;
                    n3 = 2;
                    break;
                }
                default: {
                    n = false;
                    n4 = false;
                    break;
                }
            }
        }
        if (n && JPEG.isNonStandardICC(colorSpace)) {
            n5 = true;
        }
        if (!(n == 0)) {
            JFIFMarkerSegment o = new JFIFMarkerSegment();
            this.markerSequence.add(o);
            if (!(n5 == 0)) {
                try {
                    o.addICC((ICC_ColorSpace)colorSpace);
                }
                catch (IOException ex) {}
            }
        }
        if (!(n2 == 0)) {
            this.markerSequence.add(new AdobeMarkerSegment(n3));
        }
        if (!(n8 == 0)) {
            this.markerSequence.add(new DQTMarkerSegment(compressionQuality, (boolean)n4));
        }
        if (!(n9 == 0)) {
            this.markerSequence.add(new DHTMarkerSegment((boolean)n4));
        }
        this.markerSequence.add(new SOFMarkerSegment((boolean)n6, (boolean)n7, (boolean)n4, array, n10));
        if (n6 == 0) {
            this.markerSequence.add(new SOSMarkerSegment((boolean)n4, array, n10));
        }
        if (!this.isConsistent()) {
            throw new InternalError("Default image metadata is inconsistent");
        }
        return;
    }
    
    MarkerSegment findMarkerSegment(int n) {
        for (MarkerSegment markerSegment : this.markerSequence) {
            if (markerSegment.tag == n) {
                return markerSegment;
            }
            continue;
        }
        return null;
    }
    
    MarkerSegment findMarkerSegment(Class clazz, boolean b) {
        if (b) {
            for (MarkerSegment markerSegment2 : this.markerSequence) {
                if (clazz.isInstance(markerSegment2)) {
                    return markerSegment2;
                }
                continue;
            }
        }
        else {
            ListIterator<Object> listIterator = this.markerSequence.listIterator(this.markerSequence.size());
            while (listIterator.hasPrevious()) {
                MarkerSegment markerSegment = (MarkerSegment)listIterator.previous();
                if (clazz.isInstance(markerSegment)) {
                    return markerSegment;
                }
                continue;
            }
        }
        return null;
    }
    
    private int findMarkerSegmentPosition(Class clazz, boolean b) {
        if (b) {
            ListIterator<Object> listIterator2 = this.markerSequence.listIterator();
            int n2 = 0;
            while (listIterator2.hasNext()) {
                if (clazz.isInstance((Object)listIterator2.next())) {
                    return n2;
                }
                ++n2;
            }
        }
        else {
            ListIterator<Object> listIterator = this.markerSequence.listIterator(this.markerSequence.size());
            int n = this.markerSequence.size() - 1;
            while (listIterator.hasPrevious()) {
                if (clazz.isInstance((Object)listIterator.previous())) {
                    return n;
                }
                --n;
            }
        }
        return -1;
    }
    
    private int findLastUnknownMarkerSegmentPosition() {
        ListIterator<Object> listIterator = this.markerSequence.listIterator(this.markerSequence.size());
        int n = this.markerSequence.size() - 1;
        while (listIterator.hasPrevious()) {
            if (((MarkerSegment)listIterator.previous()).unknown) {
                return n;
            }
            --n;
        }
        return -1;
    }
    
    @Override
    protected Object clone() {
        JPEGMetadata jpegMetadata = null;
        try {
            jpegMetadata = (JPEGMetadata)super.clone();
        }
        catch (CloneNotSupportedException ex) {}
        if (this.markerSequence != null) {
            jpegMetadata.markerSequence = this.cloneSequence();
        }
        jpegMetadata.resetSequence = null;
        return jpegMetadata;
    }
    
    private List cloneSequence() {
        if (this.markerSequence == null) {
            return null;
        }
        ArrayList list = new ArrayList(this.markerSequence.size());
        Iterator<Object> iterator = this.markerSequence.iterator();
        while (iterator.hasNext()) {
            list.add(((MarkerSegment)iterator.next()).clone());
        }
        return list;
    }
    
    @Override
    public Node getAsTree(String str) {
        if (str == null) {
            throw new IllegalArgumentException("null formatName!");
        }
        if (this.isStream) {
            if (str.equals("javax_imageio_jpeg_stream_1.0")) {
                return this.getNativeTree();
            }
        }
        else {
            if (str.equals("javax_imageio_jpeg_image_1.0")) {
                return this.getNativeTree();
            }
            if (str.equals("javax_imageio_1.0")) {
                return this.getStandardTree();
            }
        }
        throw new IllegalArgumentException("Unsupported format name: " + str);
    }
    
    IIOMetadataNode getNativeTree() {
        Iterator<Object> iterator = this.markerSequence.iterator();
        IIOMetadataNode iioMetadataNode;
        if (this.isStream) {
            iioMetadataNode = (iioMetadataNode = new IIOMetadataNode("javax_imageio_jpeg_stream_1.0"));
        }
        else {
            IIOMetadataNode newChild = new IIOMetadataNode("markerSequence");
            if (!this.inThumb) {
                iioMetadataNode = new IIOMetadataNode("javax_imageio_jpeg_image_1.0");
                IIOMetadataNode newChild2 = new IIOMetadataNode("JPEGvariety");
                iioMetadataNode.appendChild(newChild2);
                JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
                if (jfifMarkerSegment != null) {
                    iterator.next();
                    newChild2.appendChild(jfifMarkerSegment.getNativeNode());
                }
                iioMetadataNode.appendChild(newChild);
            }
            else {
                iioMetadataNode = newChild;
            }
            iioMetadataNode = newChild;
        }
        while (iterator.hasNext()) {
            iioMetadataNode.appendChild(((MarkerSegment)iterator.next()).getNativeNode());
        }
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardChromaNode() {
        this.hasAlpha = false;
        SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
        if (sofMarkerSegment == null) {
            return null;
        }
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Chroma");
        IIOMetadataNode newChild = new IIOMetadataNode("ColorSpaceType");
        iioMetadataNode.appendChild(newChild);
        int length = sofMarkerSegment.componentSpecs.length;
        IIOMetadataNode newChild2 = new IIOMetadataNode("NumChannels");
        iioMetadataNode.appendChild(newChild2);
        newChild2.setAttribute("value", Integer.toString(length));
        if (this.findMarkerSegment(JFIFMarkerSegment.class, true) != null) {
            if (length == 1) {
                newChild.setAttribute("name", "GRAY");
                return iioMetadataNode;
            }
            newChild.setAttribute("name", "YCbCr");
            return iioMetadataNode;
        }
        else {
            AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
            if (adobeMarkerSegment != null) {
                switch (adobeMarkerSegment.transform) {
                    case 2: {
                        newChild.setAttribute("name", "YCCK");
                        break;
                    }
                    case 1: {
                        newChild.setAttribute("name", "YCbCr");
                        break;
                    }
                    case 0: {
                        if (length == 3) {
                            newChild.setAttribute("name", "RGB");
                            break;
                        }
                        if (length == 4) {
                            newChild.setAttribute("name", "CMYK");
                            break;
                        }
                        break;
                    }
                }
                return iioMetadataNode;
            }
            if (length < 3) {
                newChild.setAttribute("name", "GRAY");
                if (length == 2) {
                    this.hasAlpha = true;
                    return iioMetadataNode;
                }
                return iioMetadataNode;
            }
            else {
                boolean n = true;
                for (int i = 0; i < sofMarkerSegment.componentSpecs.length; ++i) {
                    int componentId = sofMarkerSegment.componentSpecs[i].componentId;
                    if (componentId < 1 || componentId >= sofMarkerSegment.componentSpecs.length) {
                        n = false;
                    }
                }
                if (!(n == 0)) {
                    newChild.setAttribute("name", "YCbCr");
                    if (length == 4) {
                        this.hasAlpha = true;
                        return iioMetadataNode;
                    }
                    return iioMetadataNode;
                }
                else if (sofMarkerSegment.componentSpecs[0].componentId == 82 && sofMarkerSegment.componentSpecs[1].componentId == 71 && sofMarkerSegment.componentSpecs[2].componentId == 66) {
                    newChild.setAttribute("name", "RGB");
                    if (length == 4 && sofMarkerSegment.componentSpecs[3].componentId == 65) {
                        this.hasAlpha = true;
                        return iioMetadataNode;
                    }
                    return iioMetadataNode;
                }
                else if (sofMarkerSegment.componentSpecs[0].componentId == 89 && sofMarkerSegment.componentSpecs[1].componentId == 67 && sofMarkerSegment.componentSpecs[2].componentId == 99) {
                    newChild.setAttribute("name", "PhotoYCC");
                    if (length == 4 && sofMarkerSegment.componentSpecs[3].componentId == 65) {
                        this.hasAlpha = true;
                        return iioMetadataNode;
                    }
                    return iioMetadataNode;
                }
                else {
                    boolean n2 = false;
                    int hsamplingFactor = sofMarkerSegment.componentSpecs[0].HsamplingFactor;
                    int vsamplingFactor = sofMarkerSegment.componentSpecs[0].VsamplingFactor;
                    int j = 1;
                    while (j < sofMarkerSegment.componentSpecs.length) {
                        if (sofMarkerSegment.componentSpecs[j].HsamplingFactor != hsamplingFactor || sofMarkerSegment.componentSpecs[j].VsamplingFactor != vsamplingFactor) {
                            n2 = true;
                            break;
                        }
                        ++j;
                    }
                    if (!(n2 == 0)) {
                        newChild.setAttribute("name", "YCbCr");
                        if (length == 4) {
                            this.hasAlpha = true;
                            return iioMetadataNode;
                        }
                        return iioMetadataNode;
                    }
                    else {
                        if (length == 3) {
                            newChild.setAttribute("name", "RGB");
                            return iioMetadataNode;
                        }
                        newChild.setAttribute("name", "CMYK");
                        return iioMetadataNode;
                    }
                }
            }
        }
    }
    
    @Override
    protected IIOMetadataNode getStandardCompressionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Compression");
        IIOMetadataNode newChild = new IIOMetadataNode("CompressionTypeName");
        newChild.setAttribute("value", "JPEG");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("Lossless");
        newChild2.setAttribute("value", "FALSE");
        iioMetadataNode.appendChild(newChild2);
        int i = 0;
        Iterator<Object> iterator = this.markerSequence.iterator();
        while (iterator.hasNext()) {
            if (((MarkerSegment)iterator.next()).tag == 218) {
                ++i;
            }
            else {
                continue;
            }
        }
        if (i != 0) {
            IIOMetadataNode newChild3 = new IIOMetadataNode("NumProgressiveScans");
            newChild3.setAttribute("value", Integer.toString(i));
            iioMetadataNode.appendChild(newChild3);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardDimensionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Dimension");
        IIOMetadataNode iioMetadataNode2 = new IIOMetadataNode("ImageOrientation");
        iioMetadataNode2.setAttribute("value", "normal");
        iioMetadataNode.appendChild(iioMetadataNode2);
        JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
        if (jfifMarkerSegment != null) {
            float f;
            if (jfifMarkerSegment.resUnits == 0) {
                f = (float)jfifMarkerSegment.Xdensity / (float)jfifMarkerSegment.Ydensity;
            }
            else {
                f = (float)jfifMarkerSegment.Ydensity / (float)jfifMarkerSegment.Xdensity;
            }
            IIOMetadataNode newChild = new IIOMetadataNode("PixelAspectRatio");
            newChild.setAttribute("value", Float.toString(f));
            iioMetadataNode.insertBefore(newChild, iioMetadataNode2);
            if (jfifMarkerSegment.resUnits != 0) {
                float n = (jfifMarkerSegment.resUnits == 1) ? 25.4f : 10.0f;
                IIOMetadataNode newChild2 = new IIOMetadataNode("HorizontalPixelSize");
                newChild2.setAttribute("value", Float.toString(n / (float)jfifMarkerSegment.Xdensity));
                iioMetadataNode.appendChild(newChild2);
                IIOMetadataNode newChild3 = new IIOMetadataNode("VerticalPixelSize");
                newChild3.setAttribute("value", Float.toString(n / (float)jfifMarkerSegment.Ydensity));
                iioMetadataNode.appendChild(newChild3);
            }
        }
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardTextNode() {
        IIOMetadataNode iioMetadataNode = null;
        if (this.findMarkerSegment(254) != null) {
            iioMetadataNode = new IIOMetadataNode("Text");
            for (MarkerSegment markerSegment : this.markerSequence) {
                if (markerSegment.tag == 254) {
                    COMMarkerSegment comMarkerSegment = (COMMarkerSegment)markerSegment;
                    IIOMetadataNode newChild = new IIOMetadataNode("TextEntry");
                    newChild.setAttribute("keyword", "comment");
                    newChild.setAttribute("value", comMarkerSegment.getComment());
                    iioMetadataNode.appendChild(newChild);
                }
                else {
                    continue;
                }
            }
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardTransparencyNode() {
        IIOMetadataNode iioMetadataNode = null;
        if (this.hasAlpha) {
            iioMetadataNode = new IIOMetadataNode("Transparency");
            IIOMetadataNode newChild = new IIOMetadataNode("Alpha");
            newChild.setAttribute("value", "nonpremultiplied");
            iioMetadataNode.appendChild(newChild);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public void mergeTree(String str, Node offendingNode) throws IIOInvalidTreeException {
        if (str == null) {
            throw new IllegalArgumentException("null formatName!");
        }
        if (offendingNode == null) {
            throw new IllegalArgumentException("null root!");
        }
        List<Object> markerSequence;
        if (this.resetSequence == null) {
            this.resetSequence = this.cloneSequence();
            markerSequence = this.resetSequence;
        }
        else {
            markerSequence = this.cloneSequence();
        }
        if (this.isStream && str.equals("javax_imageio_jpeg_stream_1.0")) {
            this.mergeNativeTree(offendingNode);
        }
        else if (!this.isStream && str.equals("javax_imageio_jpeg_image_1.0")) {
            this.mergeNativeTree(offendingNode);
        }
        else if (!this.isStream && str.equals("javax_imageio_1.0")) {
            this.mergeStandardTree(offendingNode);
        }
        else {
            throw new IllegalArgumentException("Unsupported format name: " + str);
        }
        if (!this.isConsistent()) {
            this.markerSequence = markerSequence;
            throw new IIOInvalidTreeException("Merged tree is invalid; original restored", offendingNode);
        }
        return;
    }
    
    private void mergeNativeTree(Node node) throws IIOInvalidTreeException {
        String nodeName = node.getNodeName();
        if (nodeName != (this.isStream ? "javax_imageio_jpeg_stream_1.0" : "javax_imageio_jpeg_image_1.0")) {
            throw new IIOInvalidTreeException("Invalid root node name: " + nodeName, node);
        }
        if (node.getChildNodes().getLength() != 2) {
            throw new IIOInvalidTreeException("JPEGvariety and markerSequence nodes must be present", node);
        }
        this.mergeJFIFsubtree(node.getFirstChild());
        this.mergeSequenceSubtree(node.getLastChild());
        return;
    }
    
    private void mergeJFIFsubtree(Node node) throws IIOInvalidTreeException {
        if (node.getChildNodes().getLength() != 0) {
            Node firstChild = node.getFirstChild();
            JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
            if (jfifMarkerSegment != null) {
                jfifMarkerSegment.updateFromNativeNode(firstChild, false);
            }
            else {
                this.markerSequence.add(0, new JFIFMarkerSegment(firstChild));
            }
        }
    }
    
    private void mergeSequenceSubtree(Node node) throws IIOInvalidTreeException {
        NodeList childNodes = node.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); ++i) {
            Node item = childNodes.item(i);
            String nodeName = item.getNodeName();
            if (nodeName.equals("dqt")) {
                this.mergeDQTNode(item);
            }
            else if (nodeName.equals("dht")) {
                this.mergeDHTNode(item);
            }
            else if (nodeName.equals("dri")) {
                this.mergeDRINode(item);
            }
            else if (nodeName.equals("com")) {
                this.mergeCOMNode(item);
            }
            else if (nodeName.equals("app14Adobe")) {
                this.mergeAdobeNode(item);
            }
            else if (nodeName.equals("unknown")) {
                this.mergeUnknownNode(item);
            }
            else if (nodeName.equals("sof")) {
                this.mergeSOFNode(item);
            }
            else if (nodeName.equals("sos")) {
                this.mergeSOSNode(item);
            }
            else {
                throw new IIOInvalidTreeException("Invalid node: " + nodeName, item);
            }
        }
    }
    
    private void mergeDQTNode(Node node) throws IIOInvalidTreeException {
        ArrayList<MarkerSegment> list = new ArrayList();
        for (MarkerSegment e : this.markerSequence) {
            if ((e instanceof DQTMarkerSegment)) {
                list.add(e);
            }
            else {
                continue;
            }
        }
        if (!list.isEmpty()) {
            NodeList childNodes = node.getChildNodes();
            for (int i = 0; i < childNodes.getLength(); ++i) {
                Node item = childNodes.item(i);
                int attributeValue = MarkerSegment.getAttributeValue(item, null, "qtableId", 0, 3, true);
                DQTMarkerSegment dqtMarkerSegment = null;
                int n = -1;
                int j = 0;
                while (j < list.size()) {
                    DQTMarkerSegment dqtMarkerSegment2 = (DQTMarkerSegment)list.get(j);
                    int k = 0;
                    while (k < dqtMarkerSegment2.tables.size()) {
                        if (attributeValue == ((DQTMarkerSegment.Qtable)dqtMarkerSegment2.tables.get(k)).tableID) {
                            dqtMarkerSegment = dqtMarkerSegment2;
                            n = k;
                            break;
                        }
                        ++k;
                    }
                    if (dqtMarkerSegment != null) {
                        break;
                    }
                    ++j;
                }
                if (dqtMarkerSegment != null) {
                    dqtMarkerSegment.tables.set(n, dqtMarkerSegment.getQtableFromNode(item));
                }
                else {
                    DQTMarkerSegment dqtMarkerSegment3 = (DQTMarkerSegment)list.get(list.size() - 1);
                    dqtMarkerSegment3.tables.add(dqtMarkerSegment3.getQtableFromNode(item));
                }
            }
        }
        else {
            DQTMarkerSegment o = new DQTMarkerSegment(node);
            int markerSegmentPosition = this.findMarkerSegmentPosition(DHTMarkerSegment.class, true);
            int markerSegmentPosition2 = this.findMarkerSegmentPosition(SOFMarkerSegment.class, true);
            int markerSegmentPosition3 = this.findMarkerSegmentPosition(SOSMarkerSegment.class, true);
            if (markerSegmentPosition != -1) {
                this.markerSequence.add(markerSegmentPosition, o);
            }
            else if (markerSegmentPosition2 != -1) {
                this.markerSequence.add(markerSegmentPosition2, o);
            }
            else if (markerSegmentPosition3 != -1) {
                this.markerSequence.add(markerSegmentPosition3, o);
            }
            else {
                this.markerSequence.add(o);
            }
        }
    }
    
    private void mergeDHTNode(Node node) throws IIOInvalidTreeException {
        ArrayList<MarkerSegment> list = new ArrayList();
        for (MarkerSegment e : this.markerSequence) {
            if ((e instanceof DHTMarkerSegment)) {
                list.add(e);
            }
            else {
                continue;
            }
        }
        if (!list.isEmpty()) {
            NodeList childNodes = node.getChildNodes();
            for (int i = 0; i < childNodes.getLength(); ++i) {
                Node item = childNodes.item(i);
                NamedNodeMap attributes = item.getAttributes();
                int attributeValue = MarkerSegment.getAttributeValue(item, attributes, "htableId", 0, 3, true);
                int attributeValue2 = MarkerSegment.getAttributeValue(item, attributes, "class", 0, 1, true);
                DHTMarkerSegment dhtMarkerSegment = null;
                int n = -1;
                int j = 0;
                while (j < list.size()) {
                    DHTMarkerSegment dhtMarkerSegment2 = (DHTMarkerSegment)list.get(j);
                    int k = 0;
                    while (k < dhtMarkerSegment2.tables.size()) {
                        DHTMarkerSegment.Htable htable = (DHTMarkerSegment.Htable)dhtMarkerSegment2.tables.get(k);
                        if (attributeValue == htable.tableID && attributeValue2 == htable.tableClass) {
                            dhtMarkerSegment = dhtMarkerSegment2;
                            n = k;
                            break;
                        }
                        ++k;
                    }
                    if (dhtMarkerSegment != null) {
                        break;
                    }
                    ++j;
                }
                if (dhtMarkerSegment != null) {
                    dhtMarkerSegment.tables.set(n, dhtMarkerSegment.getHtableFromNode(item));
                }
                else {
                    DHTMarkerSegment dhtMarkerSegment3 = (DHTMarkerSegment)list.get(list.size() - 1);
                    dhtMarkerSegment3.tables.add(dhtMarkerSegment3.getHtableFromNode(item));
                }
            }
        }
        else {
            DHTMarkerSegment o = new DHTMarkerSegment(node);
            int markerSegmentPosition = this.findMarkerSegmentPosition(DQTMarkerSegment.class, false);
            int markerSegmentPosition2 = this.findMarkerSegmentPosition(SOFMarkerSegment.class, true);
            int markerSegmentPosition3 = this.findMarkerSegmentPosition(SOSMarkerSegment.class, true);
            if (markerSegmentPosition != -1) {
                this.markerSequence.add(markerSegmentPosition + 1, o);
            }
            else if (markerSegmentPosition2 != -1) {
                this.markerSequence.add(markerSegmentPosition2, o);
            }
            else if (markerSegmentPosition3 != -1) {
                this.markerSequence.add(markerSegmentPosition3, o);
            }
            else {
                this.markerSequence.add(o);
            }
        }
    }
    
    private void mergeDRINode(Node node) throws IIOInvalidTreeException {
        DRIMarkerSegment driMarkerSegment = (DRIMarkerSegment)this.findMarkerSegment(DRIMarkerSegment.class, true);
        if (driMarkerSegment != null) {
            driMarkerSegment.updateFromNativeNode(node, false);
        }
        else {
            DRIMarkerSegment driMarkerSegment2 = new DRIMarkerSegment(node);
            int markerSegmentPosition = this.findMarkerSegmentPosition(SOFMarkerSegment.class, true);
            int markerSegmentPosition2 = this.findMarkerSegmentPosition(SOSMarkerSegment.class, true);
            if (markerSegmentPosition != -1) {
                this.markerSequence.add(markerSegmentPosition, driMarkerSegment2);
            }
            else if (markerSegmentPosition2 != -1) {
                this.markerSequence.add(markerSegmentPosition2, driMarkerSegment2);
            }
            else {
                this.markerSequence.add(driMarkerSegment2);
            }
        }
    }
    
    private void mergeCOMNode(Node node) throws IIOInvalidTreeException {
        this.insertCOMMarkerSegment(new COMMarkerSegment(node));
    }
    
    private void insertCOMMarkerSegment(COMMarkerSegment comMarkerSegment) {
        int markerSegmentPosition = this.findMarkerSegmentPosition(COMMarkerSegment.class, false);
        int n = this.findMarkerSegment(JFIFMarkerSegment.class, true) != null;
        int markerSegmentPosition2 = this.findMarkerSegmentPosition(AdobeMarkerSegment.class, true);
        if (markerSegmentPosition != -1) {
            this.markerSequence.add(markerSegmentPosition + 1, comMarkerSegment);
        }
        else if (!(n == 0)) {
            this.markerSequence.add(1, comMarkerSegment);
        }
        else if (markerSegmentPosition2 != -1) {
            this.markerSequence.add(markerSegmentPosition2 + 1, comMarkerSegment);
        }
        else {
            this.markerSequence.add(0, comMarkerSegment);
        }
    }
    
    private void mergeAdobeNode(Node node) throws IIOInvalidTreeException {
        AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
        if (adobeMarkerSegment != null) {
            adobeMarkerSegment.updateFromNativeNode(node, false);
        }
        else {
            this.insertAdobeMarkerSegment(new AdobeMarkerSegment(node));
        }
    }
    
    private void insertAdobeMarkerSegment(AdobeMarkerSegment adobeMarkerSegment) {
        int n = this.findMarkerSegment(JFIFMarkerSegment.class, true) != null;
        int lastUnknownMarkerSegmentPosition = this.findLastUnknownMarkerSegmentPosition();
        if (!(n == 0)) {
            this.markerSequence.add(1, adobeMarkerSegment);
        }
        else if (lastUnknownMarkerSegmentPosition != -1) {
            this.markerSequence.add(lastUnknownMarkerSegmentPosition + 1, adobeMarkerSegment);
        }
        else {
            this.markerSequence.add(0, adobeMarkerSegment);
        }
    }
    
    private void mergeUnknownNode(Node node) throws IIOInvalidTreeException {
        MarkerSegment markerSegment = new MarkerSegment(node);
        int lastUnknownMarkerSegmentPosition = this.findLastUnknownMarkerSegmentPosition();
        int n = this.findMarkerSegment(JFIFMarkerSegment.class, true) != null;
        int markerSegmentPosition = this.findMarkerSegmentPosition(AdobeMarkerSegment.class, true);
        if (lastUnknownMarkerSegmentPosition != -1) {
            this.markerSequence.add(lastUnknownMarkerSegmentPosition + 1, markerSegment);
        }
        else if (!(n == 0)) {
            this.markerSequence.add(1, markerSegment);
        }
        if (markerSegmentPosition != -1) {
            this.markerSequence.add(markerSegmentPosition, markerSegment);
        }
        else {
            this.markerSequence.add(0, markerSegment);
        }
    }
    
    private void mergeSOFNode(Node node) throws IIOInvalidTreeException {
        SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
        if (sofMarkerSegment != null) {
            sofMarkerSegment.updateFromNativeNode(node, false);
        }
        else {
            SOFMarkerSegment sofMarkerSegment2 = new SOFMarkerSegment(node);
            int markerSegmentPosition = this.findMarkerSegmentPosition(SOSMarkerSegment.class, true);
            if (markerSegmentPosition != -1) {
                this.markerSequence.add(markerSegmentPosition, sofMarkerSegment2);
            }
            else {
                this.markerSequence.add(sofMarkerSegment2);
            }
        }
    }
    
    private void mergeSOSNode(Node offendingNode) throws IIOInvalidTreeException {
        SOSMarkerSegment sosMarkerSegment = (SOSMarkerSegment)this.findMarkerSegment(SOSMarkerSegment.class, true);
        SOSMarkerSegment sosMarkerSegment2 = (SOSMarkerSegment)this.findMarkerSegment(SOSMarkerSegment.class, false);
        if (sosMarkerSegment != null) {
            if (sosMarkerSegment != sosMarkerSegment2) {
                throw new IIOInvalidTreeException("Can't merge SOS node into a tree with > 1 SOS node", offendingNode);
            }
            sosMarkerSegment.updateFromNativeNode(offendingNode, false);
        }
        else {
            this.markerSequence.add(new SOSMarkerSegment(offendingNode));
        }
    }
    
    private void mergeStandardTree(Node node) throws IIOInvalidTreeException {
        this.transparencyDone = false;
        NodeList childNodes = node.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); ++i) {
            Node item = childNodes.item(i);
            String nodeName = item.getNodeName();
            if (nodeName.equals("Chroma")) {
                this.mergeStandardChromaNode(item, childNodes);
            }
            else if (nodeName.equals("Compression")) {
                this.mergeStandardCompressionNode(item);
            }
            else if (nodeName.equals("Data")) {
                this.mergeStandardDataNode(item);
            }
            else if (nodeName.equals("Dimension")) {
                this.mergeStandardDimensionNode(item);
            }
            else if (nodeName.equals("Document")) {
                this.mergeStandardDocumentNode(item);
            }
            else if (nodeName.equals("Text")) {
                this.mergeStandardTextNode(item);
            }
            else if (nodeName.equals("Transparency")) {
                this.mergeStandardTransparencyNode(item);
            }
            else {
                throw new IIOInvalidTreeException("Invalid node: " + nodeName, item);
            }
        }
    }
    
    private void mergeStandardChromaNode(Node offendingNode, NodeList list) throws IIOInvalidTreeException {
        if (this.transparencyDone) {
            throw new IIOInvalidTreeException("Transparency node must follow Chroma node", offendingNode);
        }
        Node firstChild = offendingNode.getFirstChild();
        if (firstChild == null || !firstChild.getNodeName().equals("ColorSpaceType")) {
            return;
        }
        String nodeValue = firstChild.getAttributes().getNamedItem("name").getNodeValue();
        boolean n = false;
        boolean n2 = false;
        int transform = 0;
        boolean n3 = false;
        byte[] array = { 1, 2, 3, 4 };
        int n4;
        if (nodeValue.equals("GRAY")) {
            n4 = 1;
            n = true;
        }
        else if (nodeValue.equals("YCbCr")) {
            n4 = 3;
            n = true;
            n3 = true;
        }
        else if (nodeValue.equals("PhotoYCC")) {
            n4 = 3;
            n2 = true;
            transform = 1;
            array[0] = 89;
            array[1] = 67;
            array[2] = 99;
        }
        else if (nodeValue.equals("RGB")) {
            n4 = 3;
            n2 = true;
            transform = 0;
            array[0] = 82;
            array[1] = 71;
            array[2] = 66;
        }
        else if (nodeValue.equals("XYZ") || nodeValue.equals("Lab") || nodeValue.equals("Luv") || nodeValue.equals("YxY") || nodeValue.equals("HSV") || nodeValue.equals("HLS") || nodeValue.equals("CMY") || nodeValue.equals("3CLR")) {
            n4 = 3;
        }
        else if (nodeValue.equals("YCCK")) {
            n4 = 4;
            n2 = true;
            transform = 2;
            n3 = true;
        }
        else if (nodeValue.equals("CMYK")) {
            n4 = 4;
            n2 = true;
            transform = 0;
        }
        else if (nodeValue.equals("4CLR")) {
            n4 = 4;
        }
        else {
            return;
        }
        boolean wantAlpha = false;
        int i = 0;
        while (i < list.getLength()) {
            Node item = list.item(i);
            if (item.getNodeName().equals("Transparency")) {
                wantAlpha = this.wantAlpha(item);
                break;
            }
            ++i;
        }
        if (!(wantAlpha == 0)) {
            ++n4;
            n = false;
            if (array[0] == 82) {
                array[3] = 65;
                n2 = false;
            }
        }
        JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
        AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
        SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
        SOSMarkerSegment sosMarkerSegment = (SOSMarkerSegment)this.findMarkerSegment(SOSMarkerSegment.class, true);
        if (sofMarkerSegment != null && sofMarkerSegment.tag == 194 && sofMarkerSegment.componentSpecs.length != n4 && sosMarkerSegment != null) {
            return;
        }
        if (n == 0 && jfifMarkerSegment != null) {
            this.markerSequence.remove(jfifMarkerSegment);
        }
        if (n && !this.isStream) {
            this.markerSequence.add(0, new JFIFMarkerSegment());
        }
        if (!(n2 == 0)) {
            if (adobeMarkerSegment == null && !this.isStream) {
                this.insertAdobeMarkerSegment(new AdobeMarkerSegment(transform));
            }
            else {
                adobeMarkerSegment.transform = transform;
            }
        }
        else if (adobeMarkerSegment != null) {
            this.markerSequence.remove(adobeMarkerSegment);
        }
        boolean n5 = false;
        boolean n6 = false;
        int[] array2 = { 0, 1, 1, 0 };
        int[] array3 = { 0, 0, 0, 0 };
        array3 = (n3 ? array2 : array3);
        if (sofMarkerSegment != null) {
            SOFMarkerSegment.ComponentSpec[] componentSpecs = sofMarkerSegment.componentSpecs;
            int n7 = sofMarkerSegment.tag == 194;
            this.markerSequence.set(this.markerSequence.indexOf(sofMarkerSegment), new SOFMarkerSegment((boolean)n7, false, (boolean)n3, array, n4));
            for (int j = 0; j < componentSpecs.length; ++j) {
                if (componentSpecs[j].QtableSelector != array3[j]) {
                    n5 = true;
                }
            }
            if (!(n7 == 0)) {
                boolean n8 = false;
                for (int l = 0; l < componentSpecs.length; ++l) {
                    if (array[l] != componentSpecs[l].componentId) {
                        n8 = true;
                    }
                }
                if (!(n8 == 0)) {
                    for (MarkerSegment markerSegment : this.markerSequence) {
                        if ((markerSegment instanceof SOSMarkerSegment)) {
                            SOSMarkerSegment sosMarkerSegment2 = (SOSMarkerSegment)markerSegment;
                            for (int n9 = 0; n9 < sosMarkerSegment2.componentSpecs.length; ++n9) {
                                int componentSelector = sosMarkerSegment2.componentSpecs[n9].componentSelector;
                                for (int n10 = 0; n10 < componentSpecs.length; ++n10) {
                                    if (componentSpecs[n10].componentId == componentSelector) {
                                        sosMarkerSegment2.componentSpecs[n9].componentSelector = array[n10];
                                    }
                                }
                            }
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            else if (sosMarkerSegment != null) {
                for (int k = 0; k < sosMarkerSegment.componentSpecs.length; ++k) {
                    if (sosMarkerSegment.componentSpecs[k].dcHuffTable != array3[k] || sosMarkerSegment.componentSpecs[k].acHuffTable != array3[k]) {
                        n6 = true;
                    }
                }
                this.markerSequence.set(this.markerSequence.indexOf(sosMarkerSegment), new SOSMarkerSegment((boolean)n3, array, n4));
            }
        }
        else if (this.isStream) {
            n5 = true;
            n6 = true;
        }
        if (!(n5 == 0)) {
            ArrayList o = new ArrayList();
            for (MarkerSegment markerSegment2 : this.markerSequence) {
                if ((markerSegment2 instanceof DQTMarkerSegment)) {
                    o.add(markerSegment2);
                }
                else {
                    continue;
                }
            }
            if (!o.isEmpty() && n3) {
                boolean n11 = false;
                Iterator<Object> iterator3 = o.iterator();
                while (iterator3.hasNext()) {
                    Iterator<Object> iterator4 = ((DQTMarkerSegment)iterator3.next()).tables.iterator();
                    while (iterator4.hasNext()) {
                        if (((DQTMarkerSegment.Qtable)iterator4.next()).tableID == 1) {
                            n11 = true;
                        }
                        else {
                            continue;
                        }
                    }
                }
                if (n11 == 0) {
                    DQTMarkerSegment.Qtable qtable = null;
                    Iterator<Object> iterator5 = o.iterator();
                    while (iterator5.hasNext()) {
                        for (DQTMarkerSegment.Qtable qtable2 : ((DQTMarkerSegment)iterator5.next()).tables) {
                            if (qtable2.tableID == 0) {
                                qtable = qtable2;
                            }
                            else {
                                continue;
                            }
                        }
                    }
                    DQTMarkerSegment dqtMarkerSegment = (DQTMarkerSegment)o.get(o.size() - 1);
                    dqtMarkerSegment.tables.add(dqtMarkerSegment.getChromaForLuma(qtable));
                }
            }
        }
        if (!(n6 == 0)) {
            ArrayList o2 = new ArrayList();
            for (MarkerSegment markerSegment3 : this.markerSequence) {
                if ((markerSegment3 instanceof DHTMarkerSegment)) {
                    o2.add(markerSegment3);
                }
                else {
                    continue;
                }
            }
            if (!o2.isEmpty() && n3) {
                boolean n12 = false;
                Iterator<Object> iterator8 = o2.iterator();
                while (iterator8.hasNext()) {
                    Iterator<Object> iterator9 = ((DHTMarkerSegment)iterator8.next()).tables.iterator();
                    while (iterator9.hasNext()) {
                        if (((DHTMarkerSegment.Htable)iterator9.next()).tableID == 1) {
                            n12 = true;
                        }
                        else {
                            continue;
                        }
                    }
                }
                if (n12 == 0) {
                    DHTMarkerSegment dhtMarkerSegment = (DHTMarkerSegment)o2.get(o2.size() - 1);
                    dhtMarkerSegment.addHtable(JPEGHuffmanTable.StdDCLuminance, true, 1);
                    dhtMarkerSegment.addHtable(JPEGHuffmanTable.StdACLuminance, true, 1);
                }
            }
        }
        return;
    }
    
    private boolean wantAlpha(Node node) {
        boolean n = false;
        Node firstChild = node.getFirstChild();
        if (firstChild.getNodeName().equals("Alpha") && firstChild.hasAttributes() && !firstChild.getAttributes().getNamedItem("value").getNodeValue().equals("none")) {
            n = true;
        }
        this.transparencyDone = true;
        return n;
    }
    
    private void mergeStandardCompressionNode(Node node) throws IIOInvalidTreeException {
    }
    
    private void mergeStandardDataNode(Node node) throws IIOInvalidTreeException {
    }
    
    private void mergeStandardDimensionNode(Node node) throws IIOInvalidTreeException {
        JFIFMarkerSegment jfifMarkerSegment = (MarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
        if (jfifMarkerSegment == null) {
            boolean n = false;
            SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
            if (sofMarkerSegment != null) {
                int length = sofMarkerSegment.componentSpecs.length;
                if (length == 1 || length == 3) {
                    n = true;
                    for (int i = 0; i < sofMarkerSegment.componentSpecs.length; ++i) {
                        if (sofMarkerSegment.componentSpecs[i].componentId != i + 1) {
                            n = false;
                        }
                    }
                    AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
                    if (adobeMarkerSegment != null && adobeMarkerSegment.transform != ((length != 1) ? 1 : 0)) {
                        n = false;
                    }
                }
            }
            if (!(n == 0)) {
                jfifMarkerSegment = new JFIFMarkerSegment();
                this.markerSequence.add(0, jfifMarkerSegment);
            }
        }
        if (jfifMarkerSegment != null) {
            NodeList childNodes = node.getChildNodes();
            for (int j = 0; j < childNodes.getLength(); ++j) {
                Node item = childNodes.item(j);
                NamedNodeMap attributes = item.getAttributes();
                String nodeName = item.getNodeName();
                if (nodeName.equals("PixelAspectRatio")) {
                    Point integerRatio = findIntegerRatio(Float.parseFloat(attributes.getNamedItem("value").getNodeValue()));
                    jfifMarkerSegment.resUnits = 0;
                    jfifMarkerSegment.Xdensity = integerRatio.x;
                    jfifMarkerSegment.Xdensity = integerRatio.y;
                }
                else if (nodeName.equals("HorizontalPixelSize")) {
                    int xdensity = (int)Math.round(1.0 / ((double)Float.parseFloat(attributes.getNamedItem("value").getNodeValue()) * 10.0));
                    jfifMarkerSegment.resUnits = 2;
                    jfifMarkerSegment.Xdensity = xdensity;
                }
                else if (nodeName.equals("VerticalPixelSize")) {
                    int ydensity = (int)Math.round(1.0 / ((double)Float.parseFloat(attributes.getNamedItem("value").getNodeValue()) * 10.0));
                    jfifMarkerSegment.resUnits = 2;
                    jfifMarkerSegment.Ydensity = ydensity;
                }
            }
        }
    }
    
    private static Point findIntegerRatio(float abs) {
        float n = 0.005f;
        abs = Math.abs(abs);
        if (abs <= n) {
            return new Point(1, 255);
        }
        if (abs >= 255.0f) {
            return new Point(255, 1);
        }
        boolean n2 = false;
        if ((double)abs < 1.0) {
            abs = 1.0f / abs;
            n2 = true;
        }
        int n3 = 1;
        int n4 = Math.round(abs);
        for (float n5 = Math.abs(abs - (float)n4); n5 > n; n5 = Math.abs(abs - (float)n4 / (float)n3)) {
            n4 = Math.round((float)(++n3) * abs);
        }
        return n2 ? new Point(n3, n4) : new Point(n4, n3);
    }
    
    private void mergeStandardDocumentNode(Node node) throws IIOInvalidTreeException {
    }
    
    private void mergeStandardTextNode(Node node) throws IIOInvalidTreeException {
        NodeList childNodes = node.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); ++i) {
            NamedNodeMap attributes = childNodes.item(i).getAttributes();
            Node namedItem = attributes.getNamedItem("compression");
            boolean n = true;
            if (namedItem != null && !namedItem.getNodeValue().equals("none")) {
                n = false;
            }
            if (!(n == 0)) {
                this.insertCOMMarkerSegment(new COMMarkerSegment(attributes.getNamedItem("value").getNodeValue()));
            }
        }
    }
    
    private void mergeStandardTransparencyNode(Node node) throws IIOInvalidTreeException {
        if (!this.transparencyDone && !this.isStream) {
            int wantAlpha = this.wantAlpha(node);
            JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
            AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
            SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
            SOSMarkerSegment sosMarkerSegment = (SOSMarkerSegment)this.findMarkerSegment(SOSMarkerSegment.class, true);
            if (sofMarkerSegment != null && sofMarkerSegment.tag == 194) {
                return;
            }
            if (sofMarkerSegment != null) {
                int length = sofMarkerSegment.componentSpecs.length;
                if (((length == 2 || length == 4) ? 1 : 0) != wantAlpha) {
                    if (!(wantAlpha == 0)) {
                        ++length;
                        if (jfifMarkerSegment != null) {
                            this.markerSequence.remove(jfifMarkerSegment);
                        }
                        if (adobeMarkerSegment != null) {
                            adobeMarkerSegment.transform = 0;
                        }
                        SOFMarkerSegment.ComponentSpec[] componentSpecs3 = new SOFMarkerSegment.ComponentSpec[length];
                        for (int k = 0; k < sofMarkerSegment.componentSpecs.length; ++k) {
                            componentSpecs3[k] = sofMarkerSegment.componentSpecs[k];
                        }
                        int n = (byte)(((byte)sofMarkerSegment.componentSpecs[0].componentId > 1) ? 65 : 4);
                        componentSpecs3[length - 1] = sofMarkerSegment.getComponentSpec((byte)n, sofMarkerSegment.componentSpecs[0].HsamplingFactor, sofMarkerSegment.componentSpecs[0].QtableSelector);
                        sofMarkerSegment.componentSpecs = componentSpecs3;
                        SOSMarkerSegment.ScanComponentSpec[] componentSpecs4 = new SOSMarkerSegment.ScanComponentSpec[length];
                        for (int l = 0; l < sosMarkerSegment.componentSpecs.length; ++l) {
                            componentSpecs4[l] = sosMarkerSegment.componentSpecs[l];
                        }
                        componentSpecs4[length - 1] = sosMarkerSegment.getScanComponentSpec((byte)n, 0);
                        sosMarkerSegment.componentSpecs = componentSpecs4;
                    }
                    else {
                        SOFMarkerSegment.ComponentSpec[] componentSpecs = new SOFMarkerSegment.ComponentSpec[--length];
                        for (int i = 0; i < length; ++i) {
                            componentSpecs[i] = sofMarkerSegment.componentSpecs[i];
                        }
                        sofMarkerSegment.componentSpecs = componentSpecs;
                        SOSMarkerSegment.ScanComponentSpec[] componentSpecs2 = new SOSMarkerSegment.ScanComponentSpec[length];
                        for (int j = 0; j < length; ++j) {
                            componentSpecs2[j] = sosMarkerSegment.componentSpecs[j];
                        }
                        sosMarkerSegment.componentSpecs = componentSpecs2;
                    }
                }
            }
        }
    }
    
    @Override
    public void setFromTree(String s, Node fromNativeTree) throws IIOInvalidTreeException {
        if (s == null) {
            throw new IllegalArgumentException("null formatName!");
        }
        if (fromNativeTree == null) {
            throw new IllegalArgumentException("null root!");
        }
        if (this.isStream && s.equals("javax_imageio_jpeg_stream_1.0")) {
            this.setFromNativeTree(fromNativeTree);
        }
        else if (!this.isStream && s.equals("javax_imageio_jpeg_image_1.0")) {
            this.setFromNativeTree(fromNativeTree);
        }
        else if (!this.isStream && s.equals("javax_imageio_1.0")) {
            super.setFromTree(s, fromNativeTree);
        }
        else {
            throw new IllegalArgumentException("Unsupported format name: " + s);
        }
        return;
    }
    
    private void setFromNativeTree(Node node) throws IIOInvalidTreeException {
        if (this.resetSequence == null) {
            this.resetSequence = this.markerSequence;
        }
        this.markerSequence = new ArrayList();
        String nodeName = node.getNodeName();
        if (nodeName != (this.isStream ? "javax_imageio_jpeg_stream_1.0" : "javax_imageio_jpeg_image_1.0")) {
            throw new IIOInvalidTreeException("Invalid root node name: " + nodeName, node);
        }
        if (!this.isStream) {
            if (node.getChildNodes().getLength() != 2) {
                throw new IIOInvalidTreeException("JPEGvariety and markerSequence nodes must be present", node);
            }
            Node firstChild = node.getFirstChild();
            if (firstChild.getChildNodes().getLength() != 0) {
                this.markerSequence.add(new JFIFMarkerSegment(firstChild.getFirstChild()));
            }
        }
        this.setFromMarkerSequenceNode(this.isStream ? node : node.getLastChild());
        return;
    }
    
    void setFromMarkerSequenceNode(Node node) throws IIOInvalidTreeException {
        NodeList childNodes = node.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); ++i) {
            Node item = childNodes.item(i);
            String nodeName = item.getNodeName();
            if (nodeName.equals("dqt")) {
                this.markerSequence.add(new DQTMarkerSegment(item));
            }
            else if (nodeName.equals("dht")) {
                this.markerSequence.add(new DHTMarkerSegment(item));
            }
            else if (nodeName.equals("dri")) {
                this.markerSequence.add(new DRIMarkerSegment(item));
            }
            else if (nodeName.equals("com")) {
                this.markerSequence.add(new COMMarkerSegment(item));
            }
            else if (nodeName.equals("app14Adobe")) {
                this.markerSequence.add(new AdobeMarkerSegment(item));
            }
            else if (nodeName.equals("unknown")) {
                this.markerSequence.add(new MarkerSegment(item));
            }
            else if (nodeName.equals("sof")) {
                this.markerSequence.add(new SOFMarkerSegment(item));
            }
            else if (nodeName.equals("sos")) {
                this.markerSequence.add(new SOSMarkerSegment(item));
            }
            else {
                throw new IIOInvalidTreeException("Invalid " + (this.isStream ? "stream " : "image ") + "child: " + nodeName, item);
            }
        }
    }
    
    private boolean isConsistent() {
        SOFMarkerSegment sofMarkerSegment = (SOFMarkerSegment)this.findMarkerSegment(SOFMarkerSegment.class, true);
        JFIFMarkerSegment jfifMarkerSegment = (JFIFMarkerSegment)this.findMarkerSegment(JFIFMarkerSegment.class, true);
        AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)this.findMarkerSegment(AdobeMarkerSegment.class, true);
        boolean n = true;
        if (!this.isStream) {
            if (sofMarkerSegment != null) {
                int length = sofMarkerSegment.componentSpecs.length;
                int countScanBands = this.countScanBands();
                if (countScanBands != 0 && countScanBands != length) {
                    n = false;
                }
                if (jfifMarkerSegment != null) {
                    if (length != 1 && length != 3) {
                        n = false;
                    }
                    for (int i = 0; i < length; ++i) {
                        if (sofMarkerSegment.componentSpecs[i].componentId != i + 1) {
                            n = false;
                        }
                    }
                    if (adobeMarkerSegment != null && ((length == 1 && adobeMarkerSegment.transform != 0) || (length == 3 && adobeMarkerSegment.transform != 1))) {
                        n = false;
                    }
                }
            }
            else {
                SOSMarkerSegment sosMarkerSegment = (SOSMarkerSegment)this.findMarkerSegment(SOSMarkerSegment.class, true);
                if (jfifMarkerSegment != null || adobeMarkerSegment != null || sofMarkerSegment != null || sosMarkerSegment != null) {
                    n = false;
                }
            }
        }
        return n;
    }
    
    private int countScanBands() {
        ArrayList<Object> list = new ArrayList();
        for (MarkerSegment markerSegment : this.markerSequence) {
            if ((markerSegment instanceof SOSMarkerSegment)) {
                SOSMarkerSegment.ScanComponentSpec[] componentSpecs = ((SOSMarkerSegment)markerSegment).componentSpecs;
                for (int i = 0; i < componentSpecs.length; ++i) {
                    Integer o = new Integer(componentSpecs[i].componentSelector);
                    if (!list.contains(o)) {
                        list.add(o);
                    }
                }
            }
            else {
                continue;
            }
        }
        return list.size();
    }
    
    void writeToStream(ImageOutputStream imageOutputStream, boolean b, boolean b2, List list, ICC_Profile icc_Profile, boolean b3, int transform, JPEGImageWriter jpegImageWriter) throws IOException {
        if (b2) {
            JFIFMarkerSegment.writeDefaultJFIF(imageOutputStream, list, icc_Profile, jpegImageWriter);
            if (!b3 && transform != -1 && transform != 0 && transform != 1) {
                b3 = true;
                jpegImageWriter.warningOccurred(13);
            }
        }
        for (MarkerSegment markerSegment : this.markerSequence) {
            if ((markerSegment instanceof JFIFMarkerSegment)) {
                if (!b) {
                    ((JFIFMarkerSegment)markerSegment).writeWithThumbs(imageOutputStream, list, jpegImageWriter);
                    if (icc_Profile != null) {
                        JFIFMarkerSegment.writeICC(icc_Profile, imageOutputStream);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            else if ((markerSegment instanceof AdobeMarkerSegment)) {
                if (!b3) {
                    if (transform != -1) {
                        AdobeMarkerSegment adobeMarkerSegment2 = (AdobeMarkerSegment)markerSegment.clone();
                        adobeMarkerSegment2.transform = transform;
                        adobeMarkerSegment2.write(imageOutputStream);
                    }
                    else if (b2) {
                        AdobeMarkerSegment adobeMarkerSegment = (AdobeMarkerSegment)markerSegment;
                        if (adobeMarkerSegment.transform == 0 || adobeMarkerSegment.transform == 1) {
                            adobeMarkerSegment.write(imageOutputStream);
                        }
                        else {
                            jpegImageWriter.warningOccurred(13);
                        }
                    }
                    else {
                        markerSegment.write(imageOutputStream);
                    }
                }
                else {
                    continue;
                }
            }
            else {
                markerSegment.write(imageOutputStream);
            }
        }
    }
    
    @Override
    public void reset() {
        if (this.resetSequence != null) {
            this.markerSequence = this.resetSequence;
            this.resetSequence = null;
        }
    }
    
    public void print() {
        for (int i = 0; i < this.markerSequence.size(); ++i) {
            ((MarkerSegment)this.markerSequence.get(i)).print();
        }
    }
}
